#
#  mpq.rb
#  DbcExtractor
#  This file requires you to have built the Stormlib framework and have it installed.
#  This code is using macruby bridge
#
framework 'StormLib'

require 'fileutils'

Entry = Struct.new(:path,:os_path)

class MPQ
	FIND_POINTER = "{_SFILE_FIND_DATA=[1024c]*IIIIIIII}"
	def initialize
		@mph = Pointer.new('^v')
	end
	
	def MPQ.valid_dir?(path)
		Dir.exists?("#{path}/Data") and Dir.exists?("#{path}/World of Warcraft.app")
	end
	
	def open(file)
		puts "Opening #{file}"
		raise "Unable to open archive" unless SFileOpenArchive(file,0,0x0100,@mph) == 1
		true
	end
	
	def close()
		SFileCloseArchive(@mph.value)
		true
	end
	
	def list(pattern)
		h = []
		x = Pointer.new_with_type(FIND_POINTER)
		fh = SFileFindFirstFile(@mph.value,pattern,x,nil)
		if (fh)
			path = x.value[0].pack('c*').rstrip
			os_path = path.gsub(/\\/,'/')
			entry = Entry.new(path,os_path)
			h << entry
			x = Pointer.new_with_type(FIND_POINTER)
			while SFileFindNextFile(fh,x) != 0
				path = x.value[0].pack('c*').rstrip
				os_path = path.gsub(/\\/,'/')
				entry = Entry.new(path,os_path)
				h << entry
				x = Pointer.new_with_type(FIND_POINTER)
			end
		end
		h
	end
	
	def patch(file,prefix)
		puts "Patching #{file}"
		raise "Unable to open archive" unless SFileOpenPatchArchive(@mph.value,file,nil,0) == 1
		true
	end
	
	def extract(entry,dest)
		os_file = "#{dest}/#{entry.os_path}"
		unless File.exists?(os_file)
			FileUtils.makedirs(File.dirname(os_file))
		else
			File.delete(os_file)
		end
		rc = SFileVerifyFile(@mph.value,entry.path,SFILE_VERIFY_ALL)
		if (rc & VERIFY_FILE_ERROR_MASK) == 0
			@cb.call("Creating #{os_file}") if @cb
			raise "Failed to extract #{os_file} #{GetLastError()}" unless SFileExtractFile(@mph.value,entry.path,os_file,SFILE_OPEN_PATCHED_FILE) == 1
		else
			raise "Failed to verify file #{entry.path} #{rc & VERIFY_FILE_ERROR_MASK} #{rc} #{GetLastError()}"
		end
	end
end

class Warcraft
	def initialize(path)
		@wow_path = "#{path}/Data"
	end
	
	def add_callback(&callback)
		@cb = callback
	end
	
	def load_mpqs(locale="enUS")
		mpq = MPQ.new
		mpq.open("#{@wow_path}/misc.MPQ")
		mpq.patch("#{@wow_path}/#{locale}/locale-#{locale}.MPQ",locale)
		# now add the 
		base_files = Dir.entries("#{@wow_path}")
		base_files.delete_if do |item| not item =~ /wow.*MPQ/ end
		base_files.sort.each do |b|
			x = b.gsub("base",locale)
			@cb.call "#{@wow_path}/#{b}" if @cb
			mpq.patch("#{@wow_path}/#{b}","enUS")
			if File.exists?("#{@wow_path}/#{locale}/#{x}")
				mpq.patch("#{@wow_path}/#{locale}/#{x}",locale)
			end
		end		
		return mpq
	end
	
end
if __FILE__ == $0
	wow = Warcraft.new(ARGV[0])
	m = wow.load_mpqs()
	lst = m.list("*.dbc")
	lst.each do |l|
		puts "Extracting #{l.os_path}"
		m.extract(l,ARGV[1])
	end
	m.close
end